﻿using Coscine.ApiCommons.Exceptions;
using Coscine.Configuration;
using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;

namespace Coscine.ApiCommons.Utils
{
    public class JWTHandler
    {
        private readonly IConfiguration _configuration;
        private readonly JwtSecurityTokenHandler _jwtSecurityTokenHandler;

        public JWTHandler(IConfiguration configuration)
        {
            _configuration = configuration;
            _jwtSecurityTokenHandler = new JwtSecurityTokenHandler();
        }

        public SymmetricSecurityKey GetSecurityKey()
        {
            string secretKey = _configuration.GetStringAndWait("coscine/global/jwtsecret");

            if (secretKey == null)
            {
                throw new ArgumentNullException("JWT Secret Configuration value is not set!");
            }

            return new SymmetricSecurityKey(Encoding.UTF8.GetBytes(secretKey));
        }

        public bool ValidToken(string token, string encryptionAlgorithm = "HS256")
        {

            TokenValidationParameters tokenValidationParameters = new TokenValidationParameters
            {
                IssuerSigningKey = GetSecurityKey(),
                // TODO: Validate those two
                ValidateAudience = false,
                ValidateIssuer = false
            };

            try
            {
                _jwtSecurityTokenHandler.ValidateToken(token, tokenValidationParameters, out SecurityToken securityToken);
            }
            catch (Exception)
            {
                return false;
            }
            return true;
        }

        public IEnumerable<Claim> GetContents(string jwt, string encryptionAlgorithm = "HS256")
        {
            if (ValidToken(jwt, encryptionAlgorithm))
            {
                var token = _jwtSecurityTokenHandler.ReadJwtToken(jwt);
                return token.Claims;
            }
            else
            {
                throw new InvalidTokenException("Invalid token!");
            }
        }

        public string GenerateJwtToken(JwtPayload payload, string encryptionAlgorithm = "HS256")
        {
            if (payload == null)
            {
                throw new ArgumentNullException("JwtPayload value is not set!");
            }

            var signingCredentials = new SigningCredentials(GetSecurityKey(), encryptionAlgorithm);

            var centuryBegin = new DateTime(1970, 1, 1);
            var exp = new TimeSpan(DateTime.Now.AddMinutes(30).Ticks - centuryBegin.Ticks).TotalSeconds;
            var now = new TimeSpan(DateTime.Now.Ticks - centuryBegin.Ticks).TotalSeconds;

            payload.Add("iss", "coscine");
            payload.Add("aud", "coscine");
            payload.Add("iat", (long)now);
            payload.Add("exp", (long)exp);

            var header = new JwtHeader(signingCredentials);
            var securityToken = new JwtSecurityToken(header, payload);

            return _jwtSecurityTokenHandler.WriteToken(securityToken);
        }

        public string GenerateJwtToken(IReadOnlyDictionary<string, string> payloadContents, string encryptionAlgorithm = "HS256")
        {
            var payload = new JwtPayload(payloadContents.Select(c => new Claim(c.Key, c.Value)));

            return GenerateJwtToken(payload, encryptionAlgorithm);
        }

    }
}
